% LC2 Similarity Measure
% http://campar.in.tum.de/Main/LC2Code
% 
% Chair for Computer Aided Medical Procedures (CAMP)
% Technische Universitaet Muenchen
% Written by: Bernhard Fuerst
% Email: fuerst@in.tum.de
%
% THE WORK IS FOR RESEARCH AND NON-COMMERCIAL PURPOSES ONLY.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL CAMP BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function [similarity, measure, weight] = LC2Similarity( img1, img2 )
    % img1: one channel image of size n*m
    % img2: multi-channel image of size n*m*c

    similarity =    -1;
    num =   size(img2,1)*size(img2,2);
    ids =   find(img1>0);
    v1 =    var(img1(ids));

    if (v1 > 10^-12)
        if (numel(ids) > num/2)
            img2r =         [reshape(img2, num, size(img2,3)) ones(num,1)];
            pimg2r =        pinv(img2r(ids,:));
            parameter =     pimg2r*img1(ids);
            similarity =    1 - (var(img1(ids) - (img2r(ids,:)*parameter)) / v1);
            weight =        sqrt(v1);
            measure =       weight * similarity;
        end
    end

    if (similarity == -1)
        similarity =    0;
        weight =        0;
        measure =       0;
    end
end
