% LC2 Similarity Measure
% http://campar.in.tum.de/Main/LC2Code
% 
% Chair for Computer Aided Medical Procedures (CAMP)
% Technische Universitaet Muenchen
% Written by: Bernhard Fuerst
% Email: fuerst@in.tum.de
%
% THE WORK IS FOR RESEARCH AND NON-COMMERCIAL PURPOSES ONLY.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL CAMP BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function [similarity, parameter] = LC2SimilarityPatch( img1, img2, patchsize )
    % img1: one channel image of size n*m
    % img2: multi-channel image of size n*m*c
    max_x =         size(img1,1);
    max_y =         size(img1,2);
    totalsize =     ((2*patchsize+1)^2)/2;
    parameter =     zeros(size(img2)+[0 0 1]);
    sim =           zeros(size(img1));
    mea =           zeros(size(img1));
    wei =           zeros(size(img1));

    for y = 1:max_y
        for x = 1:max_x
            patch1 = img1(max(1,x-patchsize):min(max_x,x+patchsize),...
                          max(1,y-patchsize):min(max_y,y+patchsize));
            patch2 = img2(max(1,x-patchsize):min(max_x,x+patchsize),...
                          max(1,y-patchsize):min(max_y,y+patchsize),:);

            if (numel(patch1) > totalsize)
                [sim(x,y), mea(x,y), wei(x,y)] = LC2Similarity( patch1, patch2 );
            end
        end
    end

    similarity = zeros([size(mea,3),1]);

    measure = mea(:,:);
    weights = wei(:,:);
    similarity = sum(measure(:))/sum(weights(:));
    similarity(isnan(similarity)) = 0;
end